( cd usr/lib ; rm -rf libgg.so.0 )
( cd usr/lib ; ln -sf libgg.so.0.0.7 libgg.so.0 )
( cd usr/lib ; rm -rf libgg.so )
( cd usr/lib ; ln -sf libgg.so.0.0.7 libgg.so )
( cd usr/lib ; rm -rf libgii.so.0 )
( cd usr/lib ; ln -sf libgii.so.0.0.8 libgii.so.0 )
( cd usr/lib ; rm -rf libgii.so )
( cd usr/lib ; ln -sf libgii.so.0.0.8 libgii.so )
( cd usr/lib ; rm -rf libggi.so.2 )
( cd usr/lib ; ln -sf libggi.so.2.0.1 libggi.so.2 )
( cd usr/lib ; rm -rf libggi.so )
( cd usr/lib ; ln -sf libggi.so.2.0.1 libggi.so )
( cd usr/lib ; rm -rf libvga.so.1 )
( cd usr/lib ; ln -sf libvga.so.1.99.ggi libvga.so.1 )
( cd usr/lib ; rm -rf libvga.so )
( cd usr/lib ; ln -sf libvga.so.1.99.ggi libvga.so )
